
/*################################################################################################
 *##################                                                            ##################
 *##################                                                            ##################
 *##################                   Built-in Functions                       ##################
 *##################                     Implementation                         ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*(c) YLS 1999
 * Copyright (c) 1999 YLS, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL or COMMERCIAL purposes and
 * without fee is subject to prior agreement from YLS.
 *
 * YLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. YLS SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  YLS
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */

#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <stdio.h>

#include <math.h>

#include <OKit/ErrLog.h>
#include <OKit/SysTypes.h>
#include <OKit/ObjTypes.h>
#include <OKit/FileIOMod.h>
#include <OKit/ExecMod.h>
#include <OKit/ParseMod.h>

#include "OKeval/InstSet.h"


#include "InstSet_error.c"





#include "InstSet/BltnDef_NotEq.c"
#include "InstSet/BltnDef_Mod.c"
#include "InstSet/BltnDef_And.c"
#include "InstSet/BltnDef_Die.c"
#include "InstSet/BltnDef_Mult.c"
#include "InstSet/BltnDef_Plus.c"
#include "InstSet/BltnDef_Minus.c"
#include "InstSet/BltnDef_Div.c"
#include "InstSet/BltnDef_Quot.c"
#include "InstSet/BltnDef_Less.c"
#include "InstSet/BltnDef_LessEq.c"
#include "InstSet/BltnDef_Eq.c"
#include "InstSet/BltnDef_More.c"
#include "InstSet/BltnDef_MoreEq.c"
#include "InstSet/BltnDef_Pow.c"
#include "InstSet/BltnDef_AND.c"
#include "InstSet/BltnDef_Break.c"
#include "InstSet/BltnDef_CallVar.c"
#include "InstSet/BltnDef_Cat.c"
#include "InstSet/BltnDef_Clear.c"
#include "InstSet/BltnDef_IsComp.c"
#include "InstSet/BltnDef_Dec.c"
#include "InstSet/BltnDef_DelVar.c"
#include "InstSet/BltnDef_Depth.c"
#include "InstSet/BltnDef_Drop.c"
#include "InstSet/BltnDef_DropN.c"
#include "InstSet/BltnDef_Dump.c"
#include "InstSet/BltnDef_Dup.c"
#include "InstSet/BltnDef_DupN.c"
#include "InstSet/BltnDef_Same.c"
#include "InstSet/BltnDef_Err.c"
#include "InstSet/BltnDef_Exec.c"
#include "InstSet/BltnDef_ExecVar.c"
#include "InstSet/BltnDef_Flse.c"
#include "InstSet/BltnDef_IsFltp.c"
#include "InstSet/BltnDef_Get.c"
#include "InstSet/BltnDef_Group.c"
#include "InstSet/BltnDef_GroupSize.c"
#include "InstSet/BltnDef_Help.c"
#include "InstSet/BltnDef_If.c"
#include "InstSet/BltnDef_IfElse.c"
#include "InstSet/BltnDef_Inc.c"
#include "InstSet/BltnDef_IsIntg.c"
#include "InstSet/BltnDef_IsDef.c"
#include "InstSet/BltnDef_Loop.c"
#include "InstSet/BltnDef_Noop.c"
#include "InstSet/BltnDef_NOT.c"
#include "InstSet/BltnDef_ObjectParse.c"
#include "InstSet/BltnDef_OR.c"
#include "InstSet/BltnDef_Peek.c"
#include "InstSet/BltnDef_Poke.c"
#include "InstSet/BltnDef_Pong.c"
#include "InstSet/BltnDef_Print.c"
#include "InstSet/BltnDef_PrintLn.c"
#include "InstSet/BltnDef_PrintStr.c"
#include "InstSet/BltnDef_RecallVar.c"
#include "InstSet/BltnDef_Rail.c"
#include "InstSet/BltnDef_RND.c"
#include "InstSet/BltnDef_RollStack.c"
#include "InstSet/BltnDef_RotStack.c"
#include "InstSet/BltnDef_StoreVar.c"
#include "InstSet/BltnDef_ToStrg.c"
#include "InstSet/BltnDef_IsStrg.c"
#include "InstSet/BltnDef_Swap.c"
#include "InstSet/BltnDef_True.c"
#include "InstSet/BltnDef_Undef.c"
#include "InstSet/BltnDef_Vars.c"
#include "InstSet/BltnDef_Xor.c"
#include "InstSet/BltnDef_Or.c"
#include "InstSet/BltnDef_Not.c"

#define BLTTABLESIZE 70

const unsigned long BltTable_Size = BLTTABLESIZE ;

OKit_Bltn BltTable[BLTTABLESIZE] = {

	(const OKit_Bltn) &_BltnDef_NotEq ,
	(const OKit_Bltn) &_BltnDef_Mod ,
	(const OKit_Bltn) &_BltnDef_And ,
	(const OKit_Bltn) &_BltnDef_Die ,
	(const OKit_Bltn) &_BltnDef_Mult ,
	(const OKit_Bltn) &_BltnDef_Plus ,
	(const OKit_Bltn) &_BltnDef_Minus ,
	(const OKit_Bltn) &_BltnDef_Div ,
	(const OKit_Bltn) &_BltnDef_Quot ,
	(const OKit_Bltn) &_BltnDef_Less ,
	(const OKit_Bltn) &_BltnDef_LessEq ,
	(const OKit_Bltn) &_BltnDef_Eq ,
	(const OKit_Bltn) &_BltnDef_More ,
	(const OKit_Bltn) &_BltnDef_MoreEq ,
	(const OKit_Bltn) &_BltnDef_Pow ,
	(const OKit_Bltn) &_BltnDef_AND ,
	(const OKit_Bltn) &_BltnDef_Break ,
	(const OKit_Bltn) &_BltnDef_CallVar ,
	(const OKit_Bltn) &_BltnDef_Cat ,
	(const OKit_Bltn) &_BltnDef_Clear ,
	(const OKit_Bltn) &_BltnDef_IsComp ,
	(const OKit_Bltn) &_BltnDef_Dec ,
	(const OKit_Bltn) &_BltnDef_DelVar ,
	(const OKit_Bltn) &_BltnDef_Depth ,
	(const OKit_Bltn) &_BltnDef_Drop ,
	(const OKit_Bltn) &_BltnDef_DropN ,
	(const OKit_Bltn) &_BltnDef_Dump ,
	(const OKit_Bltn) &_BltnDef_Dup ,
	(const OKit_Bltn) &_BltnDef_DupN ,
	(const OKit_Bltn) &_BltnDef_Same ,
	(const OKit_Bltn) &_BltnDef_Err ,
	(const OKit_Bltn) &_BltnDef_Exec ,
	(const OKit_Bltn) &_BltnDef_ExecVar ,
	(const OKit_Bltn) &_BltnDef_Flse ,
	(const OKit_Bltn) &_BltnDef_IsFltp ,
	(const OKit_Bltn) &_BltnDef_Get ,
	(const OKit_Bltn) &_BltnDef_Group ,
	(const OKit_Bltn) &_BltnDef_GroupSize ,
	(const OKit_Bltn) &_BltnDef_Help ,
	(const OKit_Bltn) &_BltnDef_If ,
	(const OKit_Bltn) &_BltnDef_IfElse ,
	(const OKit_Bltn) &_BltnDef_Inc ,
	(const OKit_Bltn) &_BltnDef_IsIntg ,
	(const OKit_Bltn) &_BltnDef_IsDef ,
	(const OKit_Bltn) &_BltnDef_Loop ,
	(const OKit_Bltn) &_BltnDef_Noop ,
	(const OKit_Bltn) &_BltnDef_NOT ,
	(const OKit_Bltn) &_BltnDef_ObjectParse ,
	(const OKit_Bltn) &_BltnDef_OR ,
	(const OKit_Bltn) &_BltnDef_Peek ,
	(const OKit_Bltn) &_BltnDef_Poke ,
	(const OKit_Bltn) &_BltnDef_Pong ,
	(const OKit_Bltn) &_BltnDef_Print ,
	(const OKit_Bltn) &_BltnDef_PrintLn ,
	(const OKit_Bltn) &_BltnDef_PrintStr ,
	(const OKit_Bltn) &_BltnDef_RecallVar ,
	(const OKit_Bltn) &_BltnDef_Rail ,
	(const OKit_Bltn) &_BltnDef_RND ,
	(const OKit_Bltn) &_BltnDef_RollStack ,
	(const OKit_Bltn) &_BltnDef_RotStack ,
	(const OKit_Bltn) &_BltnDef_StoreVar ,
	(const OKit_Bltn) &_BltnDef_ToStrg ,
	(const OKit_Bltn) &_BltnDef_IsStrg ,
	(const OKit_Bltn) &_BltnDef_Swap ,
	(const OKit_Bltn) &_BltnDef_True ,
	(const OKit_Bltn) &_BltnDef_Undef ,
	(const OKit_Bltn) &_BltnDef_Vars ,
	(const OKit_Bltn) &_BltnDef_Xor ,
	(const OKit_Bltn) &_BltnDef_Or ,
	(const OKit_Bltn) &_BltnDef_Not ,
} ;

