
/*################################################################################################
 *##################                                                            ##################
 *##################               Main program - RPL evaluation                ##################
 *##################                                                            ##################
 *##################                      Implementation                        ##################
 *##################                                                            ##################
 *################################################################################################
 */

/*(c) YLS 1999
 * Copyright (c) 1999 YLS, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL or COMMERCIAL purposes and
 * without fee is subject to prior agreement from YLS.
 *
 * YLS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. YLS SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES").  YLS
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <stdio.h>
#include <fcntl.h>
#include <string.h>

#include <OKit/Splash.h>
#include <OKit/ErrLog.h>
#include <OKit/ObjTypes.h>
#include <OKit/SysTypes.h>
#include <OKit/FileIOMod.h>
#include <OKit/ParseMod.h>

#include <OKeval/InstSet.h>



//////////////////////////////////////////////////////////////////////////////////////////////////
/////
/////   File loading and execution
/////


#define REPORT_ALL 0x01
#define STACK_ALL  0x02
#define PRINT_ALL  0x04
#define PRINT_COM  0x08
#define PRINT_END  0x10
#define STACK_END  0x20

 
// This is a HACK, the blank func was implemented in the OKit Parse Module...
short isblank(char chr) ;


unsigned long FileExec(char * file, unsigned long verb, OKit_Stack stk, OKit_NameSpace dom) {
	unsigned long off = 0 ;
	int fd ;
	int eof, eol ;
	unsigned long ln ;       // line number
	unsigned long buffallc ; // allocated size of buffer
	unsigned long buffused ; // really used size
	unsigned long buffline ; // line length
	unsigned long buffrd ;   // read amount
	char * buffer ;
	char * tbuffer ;

	OKit_Status ret ;
	OKit_List exe = NULL ;

	
	if( (file[0]=='-') && (file[1]==0) ) fd = dup(0) ;
	else fd = open(file,O_RDONLY) ;
	if(fd==-1) {
		fprintf(stderr,"Open failed on %s; aborting\n",file) ;
		return -1 ;
	}
	eof = 0 ;
	ln = 0 ;
	buffallc = 1 ;
	buffused = 0 ;
	buffline = 0 ;
	buffer = malloc(buffallc+1) ;
	if(buffer==NULL) {
		fprintf(stderr,"Failed allocation; aborting\n")	;
		return -1 ;
	}
	while( (!eof) || (buffused>0) ) {
		eol = 0 ;
		// line read into buffer
		while(!eol) {
			buffrd = read(fd,buffer+buffused,buffallc-buffused) ;
			if(buffrd==-1) {
				fprintf(stderr,"Read error in %s, line %lu; aborting\n",file,ln) ;
				return -1 ;
			}
			buffused += buffrd ;
			if(buffused<buffallc) {
				eof = 1 ;
				eol = 1 ;
				if( (buffused==0) || (buffer[buffused-1]!='\n') ) {
					buffer[buffused]='\n' ;
					buffused ++ ;
				}
			}
			else {
				tbuffer = realloc(buffer,(buffallc<<1)+1) ;
				if(tbuffer==NULL) {
					fprintf(stderr,"Failed allocation; aborting\n") ;
					return -1 ;
				}
				buffer = tbuffer ;
				buffallc = (buffallc<<1) ;
			}
		}
		buffline = 0 ;
		while(buffline<buffused) {
			if(buffer[buffline]=='\n') {
				ln ++ ;
				// empty line ?
				if(buffline==0) {
					eol = 1 ;
					buffline ++ ;
					break ;
				}
				// treat escape sequence, one less char
				else if(buffer[buffline-1]=='\\') {
					off = buffline ;
					while(off<buffused) {
						buffer[off-1] = buffer[off] ;
						off ++ ;
					}
					buffused -- ;
				}
				// real end of line
				else {
					eol = 1 ;
					buffline ++ ;
					break ;
				}
			}
			buffline ++ ;
		}
		// line parsing
		buffer[buffline-1] = 0 ;
		if(buffer[0]=='#') {
			if( (buffer[1]=='#') && (verb&PRINT_COM) ) {
				fprintf(stderr,">>%s\n",buffer+2) ;
			}
			off = 0 ;
			while(off<buffused-buffline) {
				buffer[off] = buffer[off+buffline] ;
				off ++ ;
			}
			buffused -= buffline ;
			continue ;
		}
		off = 0 ;
		while(isblank(buffer[off])) {
			if(buffer[off]==0) break ;
			off ++ ;
		}
		if(buffer[off]==0) {
			off = 0 ;
			while(off<buffused-buffline) {
				buffer[off] = buffer[off+buffline] ;
				off ++ ;
			}
			buffused -= buffline ;
			continue ;
		}
		errno = 0 ;
		exe = OKit_ListParse(BltTable,BltTable_Size,buffer,&off) ;
		// line evaluation
		if( (exe==NULL) && (errno==EINVAL) ) {
			fprintf(stderr,"Syntax error in %s, line %lu: %50.50s...\n",file,ln,buffer) ;
			fprintf(stderr,"...%20.20s%-20.20s...\n",buffer+((off<20)?0:(off-20)),buffer+off) ;
			fprintf(stderr,"                       ^\n") ;
			off = 0 ;
			while(off<buffused-buffline) {
				buffer[off] = buffer[off+buffline] ;
				off ++ ;
			}
			buffer[off] = 0 ;
			buffused -= buffline ;
			continue ;
		}
		OKit_ListDup(exe) ;
		errno = 0 ;
		ret = OKit_ListEval(stk,dom,exe) ;
		if(verb&REPORT_ALL) {
			fprintf(stderr,"\nEvaluated ") ;
			fflush(stderr) ;
			OKit_ListPrint(exe,2) ;
			if(ret.num) fprintf(stderr," ; error %lu at %s > %s\n",ret.num,ret.com,ret.msg) ;
			else fprintf(stderr," ; OK\n") ;
			fprintf(stderr,"\n") ;
		}
		if(verb&STACK_ALL) {
			OKit_StackFPrint(stk,2,STK_HDR,STK_FMT,STK_TLR,8) ;
		}
		if(verb&PRINT_ALL) {
			if(OKit_StackDepth(stk)>0) OKit_ObjectPrint(OKit_StackGet(stk,1),1) ;
			printf("\n") ;
		}
		OKit_ListDrop(exe) ;
		OKit_ListFree(exe) ;
		off = 0 ;
		while(off<buffused-buffline) {
			buffer[off] = buffer[off+buffline] ;
			off ++ ;
		}
		buffused -= buffline ;
	}
	free(buffer) ;
	close(fd) ;
	return ln ;
}



unsigned long StrExec(char * str, unsigned long verb, OKit_Stack stk, OKit_NameSpace dom) {
	unsigned long off = 0 ;
	OKit_Status ret ;
	OKit_List exe = NULL ;
	errno = 0 ;
	while(isblank(str[off])) {
		if(str[off]==0) break ;
		off ++ ;
	}
	if(str[off]==0) {
		fprintf(stderr,"\nEmpty string\n") ;
		return -1 ;
	}
	exe = OKit_ListParse(BltTable,BltTable_Size,str,&off) ;
	if( (exe==NULL) && (errno==EINVAL) ) {
		fprintf(stderr,"Syntax error : %50.50s...\n",str) ;
		fprintf(stderr,"...%20.20s%-20.20s...\n",str+((off<20)?0:(off-20)),str+off) ;
		fprintf(stderr,"                       ^\n") ;
		return -1 ;
	}
	OKit_ListDup(exe) ;
	ret = OKit_ListEval(stk,dom,exe) ;
	if(verb&REPORT_ALL) {
		fprintf(stderr,"\nEvaluated ") ;
		fflush(stderr) ;
		OKit_ListPrint(exe,2) ;
		if(ret.num) fprintf(stderr," ; error %lu at %s > %s\n",ret.num,ret.com,ret.msg) ;
		else fprintf(stderr," ; OK\n") ;
		fprintf(stderr,"\n") ;
	}
	if(verb&STACK_ALL) {
		OKit_StackFPrint(stk,2,STK_HDR,STK_FMT,STK_TLR,8) ;
	}
	if(verb&PRINT_ALL) {
		if(OKit_StackDepth(stk)>0) OKit_ObjectPrint(OKit_StackGet(stk,1),1) ;
		printf("\n") ;
	}
	OKit_ListDrop(exe) ;
	OKit_ListFree(exe) ;
	return 1 ;
}


//////////////////////////////////////////////////////////////////////////////////////////////////
/////
/////   Usage routines
/////



void splash(void) {
	printf("OKeval: a standard built-ins implementation and RPL interpreter, based on the OKit library.\n") ;
	printf("OKeval and the OKit library are both (C) 2000 Yann Landrin-Schweitzer, under licence LGPL.\n") ;
}

void usage(void) {
	printf("Usage: OKeval -h | -H | -H name | [options] -f file | [options] code-block ...\n") ;
	printf("      -h       : prints the help screen and returns\n") ;
}

void helpscreen() {
	printf("\n") ;
	splash() ;
	printf("\n") ;
	printf("Usage: OKeval -h | -H | -H name | [options] -f file | [options] code-block ...\n") ;
	printf("      -h       : prints this help and returns\n") ;
	printf("      -H       : prints help about supported built-ins\n") ;
	printf("      -f       : introduces a file name to be loaded\n") ;
	printf("\n") ;
	printf("    Options: a leading '-' followed by one or more characters among\n") ;
	printf("      c  *     : prints 'echo' comments in read file (started by ##)\n") ;
	printf("      C  *     : don't print 'echo' comments in read file\n") ;
	printf("      r        : prints error reports for each code-block\n") ;
	printf("      R        : don't print error reports\n") ;
	printf("      s        : prints all stack levels for each code-block\n") ;
	printf("      S        : prints all stack levels at the end of execution\n") ;
	printf("      p        : prints the first stack level for each code-block\n") ;
	printf("      P        : prints the first stack level at the end of execution\n") ;
	printf("      0        : absolutely quiet execution - usefull for stack preload\n") ;
	printf("                   or verbosity reset\n") ;
	printf("    (*: only meaningfull when used with \"-f file\")\n") ;
	printf("\n") ;
	printf("    Syntax:\n") ;
	printf("        Files must contain one code-block per line (use \\ before an \\n to wrap long lines)\n") ;
	printf("        Lines whose first character is # are comments, silently ignored, or echoed when the\n") ;
	printf("        TWO first characters are # and the -c flag is set.\n") ;
	printf("        A code-block is a space-separated list of\n") ;
	printf("      numbers  : +3.141526e00  /  -12345 \n") ;
	printf("      strings  : \"hello world\"\n") ;
	printf("      builtins : HELP  /  * \n") ;
	printf("      barewords: interpretated in order as numbers, then builtins, and strings\n") ;
	printf("                   if everything else failed\n") ;
	printf("      lists    : [ foo bar 10 2 / ]\n") ;
	printf("                   ( any of the above, separated by spaces and inside square brackets )\n") ;
	printf("\n") ;
	printf("      To get the list of known built-ins, type:\n") ;
	printf("                 OKeval -H\n") ;
	printf("\n") ;
	printf("      For specific help about some built-in, type:\n") ;
	printf("                 OKeval -H built-in_name\n") ;
	printf("\n") ;
}


void bltnabout(char * name) {
	OKit_Bltn blt ;
	unsigned long i ;
	blt = OKit_BltnByName(BltTable,BltTable_Size,name) ;
	printf("%s\n",name) ;
	if(blt==NULL) {
		printf("\tUNKNOWN\n") ;
	}
	else {
		printf("%s\n",blt->desc) ;
		printf("\n") ;
		printf("\tArguments: %u\n",blt->argc) ;
		i = blt->argc ;
		while(i>0) {
			i -- ;
			switch(OKit_BltnArgT(blt,i)) {
				case (OKit_Type)~0:
					printf("\t\t_ Any\n") ;
					break ;
				case OKit_OT_INTG:
					printf("\t\t_ OKit_Intg\n") ;
					break ;
				case OKit_OT_FLTP:
					printf("\t\t_ OKit_Fltp\n") ;
					break ;
				case OKit_OT_STRG:
					printf("\t\t_ OKit_Strg\n") ;
					break ;
				case OKit_OT_LIST:
					printf("\t\t_ OKit_List\n") ;
					break ;
				case OKit_OT_BLTN:
					printf("\t\t_ OKit_Bltn\n") ;
					break ;
				case 3:
					printf("\t\t_ OKit_Intg | OKit_Fltp\n") ;
					break ;
				default:
					printf("\t\t_ Mixed (%X)\n",OKit_BltnArgT(blt,i)) ;
			}
		}
		printf("\n") ;
	}
}


void bltnlist() {
	unsigned long i = 0 ;
	while(i<BltTable_Size) {
		printf("%s\t\t%s\n",BltTable[i]->name,BltTable[i]->desc) ;
		i ++ ;
	}
}


void version() {
	printf("OKeval %s (C) 2000 Yann Landrin-Schweitzer\n",VERSION) ;
	printf("Uses the OKit library, %s\n",OKit_copyright()) ;
}


/*
void bltnlist() {
	unsigned long i = 0 ;
	while(i<BltTable_Size) {
		bltnabout(BltTable[i]->name) ;
		printf("\tReturns:\n") ;
		printf("\t\t_ \n") ;
		printf("\n") ;
		printf("\tDescription:\n") ;
		printf("\t\t.\n") ;
		printf("\n\n") ;
		i ++ ;
	}
}
*/




//////////////////////////////////////////////////////////////////////////////////////////////////
/////
/////   MAIN
/////



int main(int argc, char * * argv) {
	int i = 1 ;
	int j = 0 ;
	int verb = REPORT_ALL|STACK_ALL|PRINT_END ;
	int nverb = 0 ;
	int fload = 0 ;
	OKit_Stack stk ;
	OKit_NameSpace dom ;

	OKit_ERRTRACEOFF() ;
//	OKit_ERRTRACEON() ;


	if(argc<2) {
		fprintf(stderr,"Too few arguments\n") ;
		usage() ;
		return 1 ;
	}

	if( (!strncmp(argv[i],"-h",2)) || (!strncmp(argv[i],"--h",3)) ) {
		helpscreen() ;
		return 0 ;
	}

	if( (!strncmp(argv[i],"-v",2)) || (!strncmp(argv[i],"--v",3)) ) {
		version() ;
		return 0 ;
	}

	OKit_BltnSort(BltTable,BltTable_Size) ;

	if( (!strncmp(argv[i],"-H",2)) || (!strncmp(argv[i],"--H",3)) ) {
		if(argc<3) {
			bltnlist() ;
		}
		else {
			i ++ ;
			while(i<argc) {
				bltnabout(argv[i]) ;
				printf("\n") ;
				i ++ ;
			}
		}
		return 0 ;
	}

	if( (!strncmp(argv[i],"-u",2)) ) {
		OKit_ERRTRACEDBG(0xFFFFFFFF) ;
		OKit_MESSAGE("main","Error Tracking active") ;
		i ++ ;
	}


	stk = OKit_StackAlloc(10) ;
	dom = OKit_NameSpaceAlloc(10) ;

	while(i<argc) {
		if( (argv[i][0] == '-') && (argv[i][1]!=0) ) {
			fload = 0 ;
			nverb = verb ;
			j = 1 ;
			while(argv[i][j]!=0) {
				switch(argv[i][j]) {
					case 'c':
						nverb |= PRINT_COM ;
						break ;
					case 'C':
						nverb &= ~PRINT_COM ;
						break ;
					case 'p':
						nverb &= ~PRINT_END ;
						nverb |= PRINT_ALL ;
						break ;
					case 'P':
						nverb &= ~PRINT_ALL ;
						nverb |= PRINT_END ;
						break ;
					case 'r':
						nverb |= REPORT_ALL ;
						break ;
					case 'R':
						nverb &= ~REPORT_ALL ;
						break ;
					case 's':
						nverb &= ~STACK_END ;
						nverb |= STACK_ALL ;
						break ;
					case 'S':
						nverb &= ~STACK_ALL ;
						nverb |= STACK_END ;
						break ;
					case '0':
						nverb = 0 ;
						break ;
					case 'f':
						fload = 1 ;
						break ;
					default:
						j = -1 ;
				}
				if(j<0) break ;
				j ++ ;
			}
			if(j>0) { // we never encountered an invalid char
				verb = nverb ;
				i ++ ;
				continue ;
			}
		} // either the arg did not begin with - or contained an invalid char
		if(fload) {
			if(i<argc) {
				FileExec(argv[i],verb,stk,dom) ;
			}
			else {
				fprintf(stderr,"\"-f\" option used without filename\n") ;
				usage() ;
				return 1 ;
			}
		}
		else {
			if(i<argc) {
				StrExec(argv[i],verb,stk,dom) ;
			}
		}
		fload = 0 ;
		i ++ ;
	}
	if(fload) {
		fprintf(stderr,"\"-f\" option used without filename\n") ;
		usage() ;
		return 1 ;
	}
	if(verb&STACK_END) {
		printf("\n") ;
		OKit_StackFPrint(stk,1,STK_HDR,STK_FMT,STK_TLR,8) ;
		printf("\n") ;
	}
	if(verb&PRINT_END) {
		if(OKit_StackDepth(stk)>0) OKit_ObjectPrint(OKit_StackGet(stk,1),1) ;
		printf("\n") ;
	}

	return 0 ;
}



